//============================================================================
// CFAG12864T2 CFAG12864U2 128x64 LCD in SPI mode
// fnt08x08.c: 8x8 Bitmapped Font
// 2017 - 06 - 15 Brent A. Crosby
// Crystalfontz America, Inc. http://www.crystalfontz.com
//============================================================================
//  This is free and unencumbered software released into the public domain.
//
//  Anyone is free to copy, modify, publish, use, compile, sell, or
//  distribute this software, either in source code form or as a compiled
//  binary, for any purpose, commercial or non-commercial, and by any
//  means.
//
//  In jurisdictions that recognize copyright laws, the author or authors
//  of this software dedicate any and all copyright interest in the
//  software to the public domain. We make this dedication for the benefit
//  of the public at large and to the detriment of our heirs and
//  successors. We intend this dedication to be an overt act of
//  relinquishment in perpetuity of all present and future rights to this
//  software under copyright law.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//  OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//  OTHER DEALINGS IN THE SOFTWARE.
//
//  For more information, please refer to <http://unlicense.org/>
//============================================================================
#include <Arduino.h>
#include "fnt08x08.h"

const unsigned char Font_08x08[96][8] PROGMEM =
  {{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, //  32 = 0x20 = " "
   {0x00,0x00,0x60,0xFA,0xFA,0x60,0x00,0x00}, //  33 = 0x21 = "!"
   {0x00,0xE0,0xE0,0x00,0xE0,0xE0,0x00,0x00}, //  34 = 0x22 = """
   {0x28,0xFE,0xFE,0x28,0xFE,0xFE,0x28,0x00}, //  35 = 0x23 = "#"
   {0x24,0x74,0x54,0xD6,0xD6,0x5C,0x48,0x00}, //  36 = 0x24 = "$"
   {0x42,0x46,0x0C,0x18,0x30,0x62,0x42,0x00}, //  37 = 0x25 = "%"
   {0x0C,0x5E,0xF2,0xB2,0xEC,0x5E,0x12,0x00}, //  38 = 0x26 = "&"
   {0x00,0x00,0x20,0xE0,0xC0,0x00,0x00,0x00}, //  39 = 0x27 = "'"
   {0x00,0x38,0x7C,0xC6,0x82,0x00,0x00,0x00}, //  40 = 0x28 = "("
   {0x00,0x82,0xC6,0x7C,0x38,0x00,0x00,0x00}, //  41 = 0x29 = ")"
   {0x54,0x7C,0x7C,0x38,0x7C,0x7C,0x54,0x00}, //  42 = 0x2A = "*"
   {0x00,0x10,0x10,0x7C,0x7C,0x10,0x10,0x00}, //  43 = 0x2B = "+"
   {0x00,0x00,0x02,0x0E,0x0C,0x00,0x00,0x00}, //  44 = 0x2C = ","
   {0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x00}, //  45 = 0x2D = "-"
   {0x00,0x00,0x06,0x06,0x06,0x00,0x00,0x00}, //  46 = 0x2E = "."
   {0x06,0x0C,0x18,0x30,0x60,0xC0,0x80,0x00}, //  47 = 0x2F = "/"
   {0x7C,0xFE,0x8E,0x9A,0xB2,0xFE,0x7C,0x00}, //  48 = 0x30 = "0"
   {0x00,0x42,0x42,0xFE,0xFE,0x02,0x02,0x00}, //  49 = 0x31 = "1"
   {0x42,0xC6,0x8E,0x9A,0xB2,0xE6,0x46,0x00}, //  50 = 0x32 = "2"
   {0x44,0xC6,0x92,0x92,0x92,0xFE,0x6C,0x00}, //  51 = 0x33 = "3"
   {0x08,0x18,0x38,0x68,0xFE,0xFE,0x08,0x00}, //  52 = 0x34 = "4"
   {0xE4,0xE6,0xA2,0xA2,0xA2,0xBE,0x9C,0x00}, //  53 = 0x35 = "5"
   {0x7C,0xFE,0x92,0x92,0x92,0xDE,0x4C,0x00}, //  54 = 0x36 = "6"
   {0xC0,0xC0,0x80,0x8E,0x9E,0xF0,0xE0,0x00}, //  55 = 0x37 = "7"
   {0x6C,0xFE,0x92,0x92,0x92,0xFE,0x6C,0x00}, //  56 = 0x38 = "8"
   {0x64,0xF6,0x92,0x92,0x92,0xFE,0x7C,0x00}, //  57 = 0x39 = "9"
   {0x00,0x00,0x00,0x66,0x66,0x66,0x00,0x00}, //  58 = 0x3A = ":"
   {0x00,0x00,0x01,0x67,0x66,0x00,0x00,0x00}, //  59 = 0x3B = ";"
   {0x00,0x10,0x38,0x6C,0xC6,0x82,0x00,0x00}, //  60 = 0x3C = "<"
   {0x24,0x24,0x24,0x24,0x24,0x24,0x24,0x00}, //  61 = 0x3D = "="
   {0x00,0x82,0xC6,0x6C,0x38,0x10,0x00,0x00}, //  62 = 0x3E = ">"
   {0x40,0xC0,0x80,0x8A,0x9A,0xF0,0x60,0x00}, //  63 = 0x3F = "?"
   {0x7C,0xFE,0x82,0x9A,0x9A,0xFA,0x72,0x00}, //  64 = 0x40 = "@"
   {0x3E,0x7E,0xC8,0x88,0xC8,0x7E,0x3E,0x00}, //  65 = 0x41 = "A"
   {0x82,0xFE,0xFE,0x92,0x92,0xFE,0x6C,0x00}, //  66 = 0x42 = "B"
   {0x38,0x7C,0xC6,0x82,0x82,0xC6,0x44,0x00}, //  67 = 0x43 = "C"
   {0x82,0xFE,0xFE,0x82,0xC6,0x7C,0x38,0x00}, //  68 = 0x44 = "D"
   {0xFE,0xFE,0x92,0x92,0x92,0x82,0xC6,0x00}, //  69 = 0x45 = "E"
   {0x82,0xFE,0xFE,0x92,0x90,0x90,0xC0,0x00}, //  70 = 0x46 = "F"
   {0x7C,0xFE,0x82,0x8A,0x8A,0xCE,0x4C,0x00}, //  71 = 0x47 = "G"
   {0xFE,0xFE,0x10,0x10,0x10,0xFE,0xFE,0x00}, //  72 = 0x48 = "H"
   {0x00,0x00,0x82,0xFE,0xFE,0x82,0x00,0x00}, //  73 = 0x49 = "I"
   {0x0C,0x0E,0x82,0xFE,0xFC,0x80,0x00,0x00}, //  74 = 0x4A = "J"
   {0xFE,0xFE,0x10,0x38,0x6C,0xC6,0x82,0x00}, //  75 = 0x4B = "K"
   {0x82,0xFE,0xFE,0x82,0x02,0x02,0x06,0x00}, //  76 = 0x4C = "L"
   {0xFE,0xFE,0x60,0x38,0x60,0xFE,0xFE,0x00}, //  77 = 0x4D = "M"
   {0xFE,0xFE,0x70,0x18,0x0C,0xFE,0xFE,0x00}, //  78 = 0x4E = "N"
   {0x7C,0xFE,0x82,0x82,0x82,0xFE,0x7C,0x00}, //  79 = 0x4F = "O"
   {0x82,0xFE,0xFE,0x92,0x90,0xF0,0x60,0x00}, //  80 = 0x50 = "P"
   {0x7C,0xFE,0x82,0x86,0x82,0xFF,0x7D,0x00}, //  81 = 0x51 = "Q"
   {0xFE,0xFE,0x90,0x98,0x9C,0xF6,0x62,0x00}, //  82 = 0x52 = "R"
   {0x64,0xF6,0x92,0x92,0x92,0xDE,0x4C,0x00}, //  83 = 0x53 = "S"
   {0x00,0xC0,0x82,0xFE,0xFE,0x82,0xC0,0x00}, //  84 = 0x54 = "T"
   {0xFC,0xFE,0x02,0x02,0x02,0xFE,0xFC,0x00}, //  85 = 0x55 = "U"
   {0xF0,0xF8,0x0C,0x06,0x0C,0xF8,0xF0,0x00}, //  86 = 0x56 = "V"
   {0xFE,0xFE,0x0C,0x38,0x0C,0xFE,0xFE,0x00}, //  87 = 0x57 = "W"
   {0x82,0xC6,0x7C,0x38,0x7C,0xC6,0x82,0x00}, //  88 = 0x58 = "X"
   {0x00,0xE0,0xF2,0x1E,0x1E,0xF2,0xE0,0x00}, //  89 = 0x59 = "Y"
   {0xC2,0x86,0x8E,0x9A,0xB2,0xE2,0xC6,0x00}, //  90 = 0x5A = "Z"
   {0x00,0xFE,0xFE,0x82,0x82,0x82,0x00,0x00}, //  91 = 0x5B = "["
   {0x80,0xC0,0x60,0x30,0x18,0x0C,0x06,0x00}, //  92 = 0x5C = "\"
   {0x00,0x82,0x82,0x82,0xFE,0xFE,0x00,0x00}, //  93 = 0x5D = "]"
   {0x10,0x30,0x60,0xC0,0x60,0x30,0x10,0x00}, //  94 = 0x5E = "^"
   {0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01}, //  95 = 0x5F = "_"
   {0x00,0x00,0xC0,0xE0,0x20,0x00,0x00,0x00}, //  96 = 0x60 = "`"
   {0x04,0x2E,0x2A,0x2A,0x3E,0x1E,0x02,0x00}, //  97 = 0x61 = "a"
   {0x82,0xFE,0xFE,0x22,0x22,0x3E,0x1C,0x00}, //  98 = 0x62 = "b"
   {0x1C,0x3E,0x22,0x22,0x22,0x36,0x14,0x00}, //  99 = 0x63 = "c"
   {0x1C,0x3E,0x22,0xA2,0xFE,0xFE,0x02,0x00}, // 100 = 0x64 = "d"
   {0x1C,0x3E,0x2A,0x2A,0x2A,0x3A,0x18,0x00}, // 101 = 0x65 = "e"
   {0x10,0x12,0x7E,0xFE,0x92,0xD0,0x40,0x00}, // 102 = 0x66 = "f"
   {0x18,0x3D,0x25,0x25,0x15,0x3F,0x3E,0x00}, // 103 = 0x67 = "g"
   {0x82,0xFE,0xFE,0x20,0x20,0x3E,0x1E,0x00}, // 104 = 0x68 = "h"
   {0x00,0x00,0x22,0xBE,0xBE,0x02,0x00,0x00}, // 105 = 0x69 = "i"
   {0x02,0x03,0x01,0x21,0xBF,0xBE,0x00,0x00}, // 106 = 0x6A = "j"
   {0x82,0xFE,0xFE,0x08,0x1C,0x36,0x22,0x00}, // 107 = 0x6B = "k"
   {0x00,0x00,0x00,0xFE,0xFE,0x02,0x00,0x00}, // 108 = 0x6C = "l"
   {0x1E,0x3E,0x30,0x1C,0x30,0x3E,0x1E,0x00}, // 109 = 0x6D = "m"
   {0x20,0x3E,0x1E,0x20,0x20,0x3E,0x1E,0x00}, // 110 = 0x6E = "n"
   {0x1C,0x3E,0x22,0x22,0x22,0x3E,0x1C,0x00}, // 111 = 0x6F = "o"
   {0x21,0x3F,0x1F,0x25,0x24,0x3C,0x18,0x00}, // 112 = 0x70 = "p"
   {0x18,0x3C,0x24,0x25,0x1F,0x3F,0x21,0x00}, // 113 = 0x71 = "q"
   {0x22,0x3E,0x1E,0x22,0x20,0x30,0x10,0x00}, // 114 = 0x72 = "r"
   {0x10,0x3A,0x2A,0x2A,0x2A,0x2E,0x04,0x00}, // 115 = 0x73 = "s"
   {0x20,0x20,0xFC,0xFE,0x22,0x26,0x04,0x00}, // 116 = 0x74 = "t"
   {0x3C,0x3E,0x02,0x02,0x3C,0x3E,0x02,0x00}, // 117 = 0x75 = "u"
   {0x30,0x38,0x0C,0x06,0x0C,0x38,0x30,0x00}, // 118 = 0x76 = "v"
   {0x3C,0x3E,0x06,0x0C,0x06,0x3E,0x3C,0x00}, // 119 = 0x77 = "w"
   {0x22,0x36,0x1C,0x08,0x1C,0x36,0x22,0x00}, // 120 = 0x78 = "x"
   {0x38,0x3D,0x05,0x05,0x09,0x3F,0x3E,0x00}, // 121 = 0x79 = "y"
   {0x32,0x26,0x2E,0x3A,0x32,0x26,0x00,0x00}, // 122 = 0x7A = "z"
   {0x00,0x10,0x10,0x7C,0xEE,0x82,0x82,0x00}, // 123 = 0x7B = "{"
   {0x00,0x00,0x00,0xEE,0xEE,0x00,0x00,0x00}, // 124 = 0x7C = "|"
   {0x00,0x82,0x82,0xEE,0x7C,0x10,0x10,0x00}, // 125 = 0x7D = "}"
   {0x40,0xC0,0x80,0xC0,0x40,0xC0,0x80,0x00}, // 126 = 0x7E = "~"
   {0x02,0x0E,0x3E,0x72,0x3E,0x0E,0x02,0x00}};// 127 = 0x7F = ""
//============================================================================

